﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using ClarionSimulation;

namespace ClarionSocket
{
	class MainClass
	{
		private bool isRunning = true;
		private Socket clientSocket;
		private ClarionAgent clarionAgent;


		public static void Main (string[] args)
		{
			MainClass m = new MainClass ();
			m.run ();
		}

		public MainClass() {
			clarionAgent = new ClarionAgent ();
			clarionAgent.Setup();
		}
			

		public void run() {
			Console.WriteLine ("Clarion Agent Started!");
			isRunning = true;

			Thread clientThread = new Thread(listen);
			clientThread.Start ();
			do {
				Console.WriteLine("Press 'q' to exit!"); 
			} while (Console.ReadKey().KeyChar != 'q');

			isRunning = false;

		}

		public void listen() {
			IPHostEntry host = Dns.Resolve( "localhost" );
			IPEndPoint serverAddress = new IPEndPoint(host.AddressList[0], 18763);

			clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			clientSocket.Connect(serverAddress);

			while (isRunning) {
				// Receiving
				byte[] rcvLenBytes = new byte[4];
				clientSocket.Receive (rcvLenBytes);

				int rcvLen = System.BitConverter.ToInt32 (rcvLenBytes, 0);
				byte[] rcvBytes = new byte[rcvLen];
				clientSocket.Receive (rcvBytes);
				String rcv = System.Text.Encoding.ASCII.GetString (rcvBytes);

				Console.WriteLine ("Client received: " + rcv);

				//Data received, let CLARION procceed with cognition!
				ClarionAgentActionType action = clarionAgent.RunOneCongnitionCycle(rcv);
				//Send back action the translated action...

				if (action == ClarionAgentActionType.GO_AHEAD) {
					send ("Go Ahead Action!");
				} else if (action == ClarionAgentActionType.ROTATE_CLOCKWISE) {
					send ("Rotate");
				} else {
					send ("Do Nothing");

				}
			}

			clientSocket.Close();
		}

		public void send(string toSend) {
			if (clientSocket != null) {
				int toSendLen = System.Text.Encoding.ASCII.GetByteCount (toSend);
				byte[] toSendBytes = System.Text.Encoding.ASCII.GetBytes (toSend);
				byte[] toSendLenBytes = System.BitConverter.GetBytes (toSendLen);
				clientSocket.Send (toSendLenBytes);
				clientSocket.Send (toSendBytes);
			}
		}
	}
}
